/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.collections.adapters;

import java.util.ListIterator;

public class TwoWayListIteratorAdapter<E1, E2> implements ListIterator<E2> {
	
	private final TwoWayAdapter<E1, E2> adapter;
	private final ListIterator<E1> backend;

	public TwoWayListIteratorAdapter(final TwoWayAdapter<E1, E2> adapter, final ListIterator<E1> backend) {
		this.adapter = adapter;
		this.backend = backend;
	}

	@Override public boolean hasNext() {
		return backend.hasNext();
	}

	@Override public E2 next() {
		return adapter.backward(backend.next());
	}

	@Override public void remove() {
		backend.remove();
	}

	@Override public void add(final E2 e) {
		backend.add(adapter.forward(e));
	}

	@Override public boolean hasPrevious() {
		return backend.hasPrevious();
	}

	@Override public int nextIndex() {
		return backend.nextIndex();
	}

	@Override public E2 previous() {
		return adapter.backward(backend.previous());
	}

	@Override public int previousIndex() {
		return backend.previousIndex();
	}

	@Override public void set(final E2 e) {
		backend.set(adapter.forward(e));
	}

}
