/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.collections.adapters;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;

import junit.framework.Assert;

import org.builtu.collections.adapters.ChainIterator;
import org.builtu.testing.BasicTestCase;
import org.junit.Before;
import org.junit.Test;


public class TestChainIterator extends BasicTestCase {
	
	private List<String> list1, list2, list3, reference;
	private ChainIterator<String> it;
	
	@Before public void createLists() {
		list1 = new ArrayList<String>();
		list2 = new ArrayList<String>();
		list3 = new ArrayList<String>();
		reference = new ArrayList<String>();
		it = new ChainIterator<String>();
	}
	
	@Test public void testEmpty() {
		assertEmpty();
	}

	@Test public void test1EmptyList() {
		it.addIterator(list1.iterator());
		assertEmpty();
	}
		
	@Test public void test2EmptyLists() {
		it.addIterator(list1.iterator());
		it.addIterator(list2.iterator());
		assertEmpty();
	}

	@Test public void test3EmptyLists() {
		it.addIterator(list1.iterator());
		it.addIterator(list2.iterator());
		it.addIterator(list3.iterator());
		assertEmpty();
	}

	@Test public void test10EmptyLists() {
		it.addIterator(list1.iterator());
		it.addIterator(list2.iterator());
		it.addIterator(list3.iterator());
		it.addIterator(list1.iterator());
		it.addIterator(list2.iterator());
		it.addIterator(list3.iterator());
		it.addIterator(list1.iterator());
		it.addIterator(list2.iterator());
		it.addIterator(list3.iterator());
		it.addIterator(list3.iterator());
		assertEmpty();
	}
	
	@Test public void test1element1list() {
		list1.add("1");
		reference.add("1");
		it.addIterator(list1.iterator());
		assertBehavesAsReference();
	}

	@Test public void test2elements1list() {
		list1.add("1");
		list1.add("2");
		reference.add("1");
		reference.add("2");
		it.addIterator(list1.iterator());
		assertBehavesAsReference();
	}

	@Test public void test1element2lists() {
		list1.add("1");
		list2.add("2");
		reference.add("1");
		reference.add("2");
		it.addIterator(list1.iterator());
		it.addIterator(list2.iterator());
		assertBehavesAsReference();
	}

	@Test public void test2elements2lists() {
		list1.add("1");
		list1.add("2");
		list2.add("3");
		list2.add("4");
		reference.add("1");
		reference.add("2");
		reference.add("3");
		reference.add("4");
		it.addIterator(list1.iterator());
		it.addIterator(list2.iterator());
		assertBehavesAsReference();
	}

	@Test public void test3lists() {
		list1.add("1");
		list2.add("2");
		list2.add("3");
		list3.add("4");
		list3.add("5");
		list3.add("6");
		reference.add("1");
		reference.add("2");
		reference.add("3");
		reference.add("4");
		reference.add("5");
		reference.add("6");
		it.addIterator(list1.iterator());
		it.addIterator(list2.iterator());
		it.addIterator(list3.iterator());
		assertBehavesAsReference();
	}
	
	private void assertEmpty() {
		Assert.assertFalse(it.hasNext());
		try {
			it.next();
			Assert.fail();
		} catch (final NoSuchElementException e) {}
		try {
			it.remove();
			Assert.fail();
		} catch (final IllegalStateException e) {}
	}
	
	private void assertBehavesAsReference() {
		final Iterator<String> ref = reference.iterator();
		
		try {
			ref.remove();
			Assert.fail();
		} catch (final IllegalStateException e) {}		
		
		try {
			it.remove();
			Assert.fail();
		} catch (final IllegalStateException e) {}

		for (int i=0; i<=reference.size()+100; i++) {
			final boolean ref_hasNext = ref.hasNext();
			final boolean act_hasNext = it.hasNext();
			Assert.assertEquals("i="+i, ref_hasNext, act_hasNext);
			String ref_elem = null;
			try {
				ref_elem = ref.next();
			} catch (final NoSuchElementException e) {
				try {
					it.next();
					Assert.fail("No elements in reference, but some elements in actual");
				} catch (final NoSuchElementException e2) {
					continue;
				}
			}
			String act_elem = it.next();
			Assert.assertSame(ref_elem, act_elem);
			if (rnd(3)==0) {
				ref.remove();
				it.remove();
			}
		}
	}

}
