/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.collections.bytearray;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;

import org.builtu.collections.bytearray.ByteArray;
import org.builtu.testing.BasicTestCase;
import org.junit.Test;


public class TestByteArrayStreams extends BasicTestCase {
  @Test public void testReadAll() throws IOException {
    doTestReading("Hello, world!");
  }
  
  @Test public void testRandom() throws IOException {
    doTestReading(rndAlphaString());
  }
  
  private void doTestReading(final String data) throws IOException {
    final ByteArrayInputStream in = 
      new ByteArrayInputStream(data.getBytes("UTF-8"));
      final ByteArray b = new ByteArray();
      b.readAll(in);
      assertEquals(data, b.asUTF8());
  }
  
  @Test public void testWrite() 
    throws UnsupportedEncodingException, IOException 
  {
    testAsOutputStream("Hello, world");
  }

  @Test public void testWriteRandom() 
    throws UnsupportedEncodingException, IOException 
  {
    testAsOutputStream(rndAlphaString());
  }
  
  public void testAsOutputStream(final String data) 
    throws UnsupportedEncodingException, IOException 
  {
    final ByteArray b = new ByteArray();
    final OutputStream out = b.asOutputStream();
    out.write(data.getBytes("UTF-8"));
    assertEquals(data, b.asUTF8());
  }
}
