/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.config.module.refresh;

/**
 * Refreshes when some period of time has passed.
 * @author borisman
 *
 */
public class SimpleRefreshStrategy extends AbstractRefreshStrategyBase implements RefreshStrategy {

	public static final int NEVER = -1;
	public static final int ALWAYS = 0;
	public static final int EVERY_MINUTE = 60 * 1000;
	public static final int EVERY_HOUR = 60 * 60 * 1000;
	public static final int EVERY_DAY = 24 * 60 * 60 * 1000;
	
	private int refreshMsecs = ALWAYS;

	private volatile long lastrefresh;
	
	public int getRefreshMsecs() {
		return refreshMsecs;
	}

	public void setRefreshMsecs(final int refreshMsecs) {
		this.refreshMsecs = refreshMsecs;
	}
	
	@Override public boolean hasToBeRefreshed() {
		if (refreshMsecs < 0) return false; 
		return clock.currentTimeMillis()-lastrefresh >= refreshMsecs;
	}

	@Override public void refreshed() {
		lastrefresh = clock.currentTimeMillis();
	}

}
