/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.config.module;

import java.io.IOException;

import junit.framework.Assert;

import org.builtu.config.module.Config;
import org.builtu.util.commutil.ClasspathUtil;
import org.junit.Test;



public class TestDump {
	
	@Test public void testSimple() throws IOException {
		final Config simple = new Config("/TestConfig_simple.xml");
		final String xml1 = simple.dumpKeyAsXml("module.key");
		final String xml2 = simple.dumpKeyAsXml("module");
		Assert.assertEquals(xml1, xml2);
		assertSameXmlAsResource("/TestConfig_simple_defaults.xml", xml1);
	}
	
	@Test public void testComplexFull() throws IOException {
/*		final Config simple = new Config("/TestDump_complex.xml");
		final String xml = simple.dumpKeyAsXml("module1");
		assertSameXmlAsResource("/TestDump_complex_defaults.xml", xml);*/
	}
	
	@Test public void testComplexComplexKey3() throws IOException {
		final Config simple = new Config("/TestDump_complex.xml");
		final String xml = simple.dumpKeyAsXml("module1.complexKey3");
		assertSameXml("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>" +
				        "<configuration>" +
				          "<module1>" +
				          	"<complexKey3>"+
				          		"<subkey1>"+
				          			"subvalue1"+
				          		"</subkey1>"+
				          		"<subkey2>"+
				          			"subvalue2"+
				          		"</subkey2>"+
				          	"</complexKey3>" +
				          "</module1>" +
				        "</configuration>", xml);
	}

	private static void assertSameXmlAsResource(final String classpath, final String xml) throws IOException {
		final String expected = ClasspathUtil.readUTF8ClassResource(classpath);
		assertSameXml(expected, xml);
	}

	private static void assertSameXml(String expected, String xml) {
		expected = normalize(expected);
		xml = normalize(xml);
		Assert.assertEquals(expected, xml);
	}

	private static String normalize(String xml) {
		xml = xml.replaceAll("\\s+", "").replaceAll("<!--.*?-->", "");
		return xml;
	}
}
