/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.config.module.refresh;

import junit.framework.Assert;

import org.builtu.config.module.refresh.RefreshSingleton;
import org.builtu.config.module.refresh.SimpleRefreshStrategy;
import org.junit.Before;
import org.junit.Test;



public class TestRefreshingSingletonSimple extends ClockedTestBase {
	
	private static class SimpleSingleton extends RefreshSingleton<Object> {
		@Override protected Object create() {
			return new Object();
		}
	}
	
	private SimpleSingleton s;
	private SimpleRefreshStrategy strategy; 
	
	@Before public void create() {
		s = new SimpleSingleton();
		strategy = new SimpleRefreshStrategy();
		strategy.setClock(clock);
		s.setStrategy(strategy);
	}
	
	@Test public void testRefreshAlways() {
		strategy.setRefreshMsecs(SimpleRefreshStrategy.ALWAYS);
		final Object o1 = s.get();
		final Object o2 = s.get();
		Assert.assertNotSame(o1, o2);
	}

	@Test public void testRefreshNever() {
		strategy.setRefreshMsecs(SimpleRefreshStrategy.NEVER);
		final Object o1 = s.get();
		final Object o2 = s.get();
		Assert.assertSame(o1, o2);
	}
	
	@Test public void testRefresh3seconds() throws InterruptedException {
		strategy.setRefreshMsecs(3000);
		final Object o1 = s.get();
		final Object o2 = s.get();
		Assert.assertSame(o1, o2);
		time += 3000;
		final Object o3 = s.get();
		Assert.assertNotSame(o1, o3);
	}
	
}
