package org.builtu.config.spring;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.builtu.exceptions.FatalError;
import org.builtu.util.commutil.StringUtil;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.BeanFactoryAware;
import org.springframework.beans.factory.NoSuchBeanDefinitionException;


public class FallbackBean implements BeanFactoryAware {
	
	private static final Logger log = Logger.getLogger(FallbackBean.class); 

	private BeanFactory beanFactory;
	private String lookup;
	
	private static class LookupRule {
		public final String beanName;
		public final Level logLevel;
		public final String message;

		public LookupRule(final String s) {
			final List<String> p = StringUtil.split_list(s, ':');
			if (p.size()==1) {
				this.beanName = s;
				this.logLevel = null;
				this.message = null;
			} else if (p.size()==3) {
				this.beanName = p.get(0).trim();
				this.logLevel = Level.toLevel(p.get(1).trim());
				this.message = p.get(2).trim();
			} else {
				throw new FatalError("Malformed rule '"+s+"'");
			}
		}
	}
	
	private List<LookupRule> rules;
	
	public String getLookup() {
		return lookup;
	}

	public void setLookup(final String lookup) {
		this.lookup = lookup;
		final List<String> rulesStrings = StringUtil.split_list(lookup, ',');
		this.rules = new ArrayList<LookupRule>(rulesStrings.size());
		for (String s : rulesStrings) {
			s = s.trim();
			if (s.length()==0) continue;
			final LookupRule r = new LookupRule(s); 
			rules.add(r);
		}
		log.debug(lookup);
	}

	@Override public void setBeanFactory(BeanFactory beanFactory) throws BeansException {
		this.beanFactory = beanFactory;
	}
	
	public Object getObject() {
		for (final LookupRule r : rules) {
			if (!beanFactory.containsBean(r.beanName)) continue;
			
			final Object res = beanFactory.getBean(r.beanName);

			if (res == null) {
				log.warn("Bean "+r.beanName+" was found, but it's value is null, contiuing lookup");
				continue;
			}
			
			log.log(r.logLevel, r.message);
			return res;
		}
		
		throw new NoSuchBeanDefinitionException(lookup);
	}
	
	public Object makeNullObject() {
		return null;
	}
	
}
