/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.services.resources.servlet;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.builtu.mime.MimeUtil;
import org.builtu.services.resources.Resource;
import org.builtu.services.resources.ResourceRoot;


public class ResourceRootServlet extends HttpServlet {

	private final ResourceRoot root = new ResourceRoot();
	
	@Override protected void doGet(final HttpServletRequest req, final HttpServletResponse resp)
			throws ServletException, IOException 
	{
		run(req, resp);
	}

	@Override protected void doPost(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException 
	{
		run(req, resp);
	}

	private void run(final HttpServletRequest req, final HttpServletResponse resp)
			throws ServletException, IOException 
	{
		String path = req.getPathInfo();
		if (path.length()==0) {
			resp.sendError(404);
			return;
		}
		if (path.charAt(0)!='/') path='/'+path;
		
		if (hasWWWHacks(path)) {
			resp.sendError(404);
			return;
		}
		
		final Resource res = root.get(path);
		if (res == null) {
			resp.sendError(404);
		} else {
			serve(path, res, resp);
		}
	}

	private static boolean hasWWWHacks(final String path) {
		if (path.indexOf("./")>=0) return true;
		return false;
	}

	private static void serve(final String path, final Resource res, final HttpServletResponse resp) 
		throws IOException 
	{
		final byte buffer[] = new byte[4096];
		resp.setContentLength((int)res.size());
		final InputStream in = res.openStream();
		try {
			int read = in.read(buffer);
			String mimeType = MimeUtil.getMimeTypeByName(path);
			if (mimeType == null) {
				mimeType = MimeUtil.getMimeType(buffer);
			}
			resp.setContentType(mimeType);
			final OutputStream out = resp.getOutputStream();
			try {
				while (read>=0) {
					if (read !=0) out.write(buffer, 0, read);
					read = in.read(buffer);
				}
			} finally {
				out.close();
			}
		} finally {
			in.close();
		}
	}

	private static final long serialVersionUID = 1L;
}
