/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.services.resources;

import java.io.File;

import org.apache.log4j.Logger;

public class FileResourceLocation implements ResourceLocation {
	
	private static final Logger log = Logger.getLogger(FileResourceLocation.class);
	
	private final File rootDir;
	
	public FileResourceLocation(final File rootDir) {
		assert rootDir.isDirectory() : rootDir+" is not a directory";
		this.rootDir = rootDir;
	}

	@Override public Resource resolve(final String path) {
		final File file = new File(rootDir, path);
		if (!file.isFile()) {
			log.debug("File not found: "+file);
			return null;
		}
		log.info("File found: "+file);
		return new FileResource(file);
	}

}
