/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.testing.webmock;

import java.io.InputStream;

import junit.framework.Assert;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.builtu.testing.webmock.Request;
import org.builtu.testing.webmock.Result;
import org.builtu.testing.webmock.WebMock;
import org.builtu.util.commutil.IOUtil;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;



public class TestWebMock extends AbstractWebMockTest {
	
	@Before public void start() throws Exception {
		mock = new WebMock ();
		startMock();
	}
	
	@After public void stop() throws Exception {
		mock.stop();
	}	
	
	@Test public void testStartStop() throws Exception {
	}
	
	@Test public void test404() throws Exception {
		final HttpClient httpclient = new DefaultHttpClient();
		final HttpGet httpget = new HttpGet(host());
		final HttpResponse response = httpclient.execute(httpget);
		
		Assert.assertEquals(404, response.getStatusLine().getStatusCode());
	}
	
	@Test public void testSimple() throws Exception {
		final Request request = new Request();
		request.setPath("/");
		final Result result = Result.html("Hello");
		mock.addMapping(request, result);
		
		final HttpClient httpclient = new DefaultHttpClient();
		final HttpGet httpget = new HttpGet(host());
		final HttpResponse response = httpclient.execute(httpget);
		
		Assert.assertEquals(200, response.getStatusLine().getStatusCode());
		Assert.assertEquals("text/html; charset=utf-8", response.getEntity().getContentType().getValue());
		final String contents;
		final InputStream in = response.getEntity().getContent();
		try {
			contents = IOUtil.readUTF8Contents(in);
		} finally {
			in.close();
		}
		Assert.assertEquals("Hello", contents); 
	}
}
