/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.testing.webmock;

import java.io.IOException;
import java.io.InputStream;

import junit.framework.Assert;

import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.builtu.testing.webmock.WebMock;
import org.builtu.util.commutil.IOUtil;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;



public class TestWebMockXML extends AbstractWebMockTest {
	
	@Before public void start() throws Exception {
		mock = new WebMock ();
		mock.readConfig(this.getClass().getResource("/webmock.xml"));
		startMock();
	}
	
	@After public void stop() throws Exception {
		mock.stop();
	}
	
	@Test public void testRoot() throws ClientProtocolException, IOException {
		Assert.assertEquals("<h1>HELLO</h1>", read("/"));
	}

	@Test public void testAlias() throws ClientProtocolException, IOException {
		Assert.assertEquals("<h1>HELLO</h1>", read("/index.html"));
	}

	@Test public void test_params() throws ClientProtocolException, IOException {
		Assert.assertEquals("Params should work 1",  read("/test_params?param=1"));
		Assert.assertEquals("Params should work 2",  read("/test_params?param=2"));
	}
	
	@Test public void test_ignores() throws ClientProtocolException, IOException {
		Assert.assertEquals("Ignores could be specified so that some parameters are ignored", 
				            read("/test_ignores?param1=value1&param2=value2"));
	}
	
	@Test public void test_url() throws ClientProtocolException, IOException {
		Assert.assertEquals("url params could be specified in url directly and their order does not matter", 
				            read("/test_url?param1=value1&param2=value2"));
	}
	
	@Test public void test_full_url() throws ClientProtocolException, IOException {
		Assert.assertEquals("You can even use localhost in url, it will be silently ignored", 
				            read("/test_full_url?param2=value2&param1=value"));
	}

	@Test public void test_someserver_url() throws ClientProtocolException, IOException {
		Assert.assertEquals("In fact you can use any host you like, it will be ignored", 
				            read("/test_someserver_url?param2=value2&param1=value"));
	}

	@Test public void test_twoparams() throws ClientProtocolException, IOException {
		Assert.assertEquals("You can specify parameters both in url and as tags, they will be merged", 
				            read("/test_twoparams?param1=value1&param1=value2"));
	}
	
	@Test public void test_ignores_url() throws ClientProtocolException, IOException {
		Assert.assertEquals("If you don't write '=' that means ANY value, if you do, that means empty string value", 
				            read("/test_ignores_url?param1=12312&param2="));
	}
	
	
	private String read(final String path) throws ClientProtocolException, IOException {
		final HttpClient httpclient = new DefaultHttpClient();
		final HttpGet httpget = new HttpGet(host()+path);
		final HttpResponse response = httpclient.execute(httpget);
		
		Assert.assertEquals(200, response.getStatusLine().getStatusCode());
		Assert.assertEquals("text/html; charset=utf-8", response.getEntity().getContentType().getValue());

		final InputStream in = response.getEntity().getContent();
		try {
			return IOUtil.readUTF8Contents(in);
		} finally {
			in.close();
		}
	}

}
