/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.util.clown;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

abstract class ComplexClassBase implements Serializable {
	private byte fByte;
	private int fInt;
	private float fFloat;
	private String fString;
	private List<String> fList = new ArrayList<String>();
	private List<ComplexClassBase> ref = new ArrayList<ComplexClassBase>();
	
	protected ComplexClassBase() {}
	
	public ComplexClassBase(final Random rnd) {
		this(rnd, 0);
	}

	protected ComplexClassBase(final Random rnd, final int level) {
		this.fByte = (byte)rnd.nextInt();
		this.fInt = rnd.nextInt();
		this.fFloat = rnd.nextFloat();
		this.fString = rnd.nextInt(2)==0 ? null : String.valueOf(rnd.nextInt());
		
		final int L = rnd.nextInt(10);
		for (int i=0; i<L; i++) {
			final String s = rnd.nextInt(2)==0 ? null : String.valueOf(rnd.nextInt());
			fList.add(s);
		}
		
		if (level<10) {
			final int S = rnd.nextInt(10-level);
			for (int i=0; i<S; i++) {
				final ComplexClassBase c = rnd.nextInt(2)==0 ? null : makeInstance(rnd, level+1);
				ref.add(c);
			}
		}
	}
	
	protected abstract ComplexClassBase makeInstance(final Random rnd, final int level);

	@Override public boolean equals(final Object obj) {
		if (obj == this) return false; // This is violation, but we need to check this is really is a deep copy
		if (!(obj instanceof ComplexClassBase)) return false;
		final ComplexClassBase other = (ComplexClassBase) obj;

		return this.fByte == other.fByte && 
		       this.fFloat == other.fFloat &&
		       this.fInt == other.fInt &&
		       equal(this.fString, other.fString) &&
		       arraysEqual(this.fList, other.fList) &&
		       arraysEqual(this.ref, other.ref);
	}
	
	public static <E> boolean arraysEqual(final List<E> expected, final List<E> actual) {
		if (expected.size() != actual.size()) return false;
		for (int i = 0; i < expected.size(); i++) {
			if (!equal(expected.get(i), actual.get(i))) return false;
		}
		return true;
	}
	
	public static boolean equal (final Object o1, final Object o2) {
		if (o1 == null) {
			return o2 == null;
		} else {
			return o1.equals(o2);
		}
	}

	private static final long serialVersionUID = 1L;
}
