/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.util.fs;

import java.io.File;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.regex.Pattern;

public class FindIterator implements Iterator<String> {
	
	private final Pattern pat;
	private FindIterator subiterator;
	private String base;
	private final File dir;
	private String [] currentDirContents;
	private String next;
	private int nextIdx=0;

	public FindIterator(final File dir, final Pattern pat) {
		this(dir, "", pat);
	}

	private FindIterator(final File dir, final String base, final Pattern pat) {
		this.dir = dir;
		this.pat = pat;
		this.base = base;
		this.currentDirContents = new File(dir, base).list();
	}

	@Override public boolean hasNext() {
		if (subiterator!=null && subiterator.hasNext()) return true;
		if (next != null) return true;
		return findNext();
	}

	@Override public String next() {
		if (!hasNext()) throw new NoSuchElementException();
		if (subiterator!=null && subiterator.hasNext()) return subiterator.next();
		assert next != null;
		final String res = base + next; 
		next = null;
		return res;
	}

	@Override public void remove() {
		throw new UnsupportedOperationException();
	}

	private boolean findNext() {
		while (currentDirContents != null && nextIdx<currentDirContents.length) {
			final String name = currentDirContents[nextIdx++];
			final File candidate = new File(dir, base+name);
			if (candidate.isDirectory()) {
				final FindIterator sub = new FindIterator(dir, base+name+File.separatorChar, pat);
				if (sub.hasNext()) {
					subiterator = sub;
					return true;
				}
			} else if (pat.matcher(name).matches()) {
				next = name;
				return true;
			}
		}
		return false;
	}

}
