/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.util.html;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.builtu.util.html.HtmlUtil;
import org.junit.Assert;
import org.junit.Test;


public class TestHtmlUtil {
    
	@Test public void testEmpty() {
        Assert.assertEquals("", HtmlUtil.chopBody(""));
    }

    @Test public void testEmptyhtml() {
        Assert.assertEquals("", HtmlUtil.chopBody("<html><body></body></html>"));
    }

    @Test public void testHELLO() {
        Assert.assertEquals("HELLO", HtmlUtil.chopBody("<html><body>HELLO</body></html>"));
    }

    @Test  public void testNoHtml() {
        Assert.assertEquals("HELLO", HtmlUtil.chopBody("<html>HELLO</body></html>"));
    }

    @Test public void testNoEnd() {
        Assert.assertEquals("HELLO", HtmlUtil.chopBody("<html><body>HELLO"));
    }

    @Test public void testTags() {
        Assert.assertEquals("<tag><mag><shmag>", HtmlUtil.chopBody("<html><body><tag><mag><shmag>"));
    }

    @Test public void testEscape_lt() {
        Assert.assertEquals("&lt;", HtmlUtil.escape("<"));
    }

    @Test public void testEscape_gt() {
        Assert.assertEquals("&gt;", HtmlUtil.escape(">"));
    }

    @Test public void testEscape_amp() {
        Assert.assertEquals("&amp;", HtmlUtil.escape("&"));
    }

    @Test public void testEscapeHtml() {
        Assert.assertEquals("&lt;a href='hello?a=b&amp;b=c&gt;a&lt;/a&gt;",
                HtmlUtil.escape("<a href='hello?a=b&b=c>a</a>"));
    }

    @Test public void testUnescape_lt() {
        Assert.assertEquals("<", HtmlUtil.unescape("&lt;"));
    }

    @Test public void testUnescape_gt() {
        Assert.assertEquals(">", HtmlUtil.unescape("&gt;"));
    }

    @Test public void testUnescape_amp() {
        Assert.assertEquals("&", HtmlUtil.unescape("&amp;"));
    }

    @Test public void testUnescape_nbsp() {
        Assert.assertEquals(" ", HtmlUtil.unescape("&nbsp;"));
    }

    @Test public void testUnescapeHtml() {
        Assert.assertEquals("<a href='hello?a=b&b=c>a</a>",
                HtmlUtil.unescape("&lt;a href='hello?a=b&amp;b=c&gt;a&lt;/a&gt;"));
    }

    @Test public void testStripTags() {
        Assert.assertEquals(" text1 text2 text3", HtmlUtil.stripTags("<tag1>text1</tag1>text2<tag2>text3"));
    }

    @Test public void testCanonize() {
        Assert.assertEquals("<tag1>text<tag2>text text<tag3>", 
        		HtmlUtil.canonize("<tag1> text<tag2> text text\n\r\t\f <tag3> "));
    }
    
    @Test public void testHtml2text() {
        String html =
                "<script type=\"text/javascript\" src=\"/js/jquery.js\"></script>\n" +
                "<script type=\"text/javascript\" src=\"/js/jqarousel.js\"></script>\n" +
                " <script type=\"text/javascript\">\n" +
                "\n" +
                "\n" +
                "jQuery.noConflict()(function(){\n" +
                "    \n" +
                "  jQuery('#mycarousel').jcarousel({\n" +
                "    scroll: 2,\n" +
                "    animation: 'normal'\n" +
                "   });\n" +
                "\n" +
                "  jQuery(\"#mycarousel li a\").click(function(){\n" +
                "  jQuery(\"#large img\").hide().attr({\"src\": jQuery(this).attr(\"href\"), \"title\": jQuery(\"> img\", this).attr(\"title\")});\n" +
                "  return false;\n" +
                "  });\n" +
                "  \n" +
                "  jQuery(\"#large>img\").load(function(){jQuery(\"#large>img:hidden\").show(\"fast\")});\n" +
                "   \n" +
                "   \n" +
                "  jQuery('#popup .close').click(function(){\n" +
                "  jQuery(\"#popup\").css('display', 'none');\n" +
                "  });\n" +
                "}); \n" +
                " </script>\n" +
                "<div class=\"info-page-output\">\n" +
                "\t<div id=\"info-page-gallery\">\n" +
                "\t<div id=\"photo\">\n" +
                "\t\t\t\t<img alt=\"\" src=\"/infopages/16884/5206145/5206145.1_1.jpg\" />\n" +
                "\t</div>\n" +
                "</div>\n" +
                "<h3>«Индустрия Чистоты» предлагает своим клиентам следующие виды услуг по профессиональной уборке:</h3>\n" +
                "<ul>\n" +
                "  <li>Ежедневная уборка  </li>\n" +
                "  <li>Генеральная  уборка помещений различного назначения  </li>\n" +
                "  <li>Уборка квартир, коттеджей  </li>\n" +
                "  <li>Уборка после пожара  </li>\n" +
                "  <li>Послестроительная уборка  </li>\n" +
                "  <li>Химчистка ковров  </li>\n" +
                "  <li>Химчистка мягкой мебели  </li>\n" +
                "  <li>Чистка жалюзи  </li>\n" +
                "  <li>Мытье окон  </li>\n" +
                "  <li>Мытье фасадов  </li>\n" +
                "  <li>Глубокая чистка и защита напольных покрытий  </li>\n" +
                "  <li>Чистка саун и бассейнов  </li>\n" +
                "</ul>\n" +
                "<h3>Ждем Ваших звонков по телефонам: 8-901-313-21-32 (круглосуточно), 492-94-18, 492-93-98</h3>\n" +
                "\n" +
                "</div>\n" +
                "<!-- \n" +
                "<div class=\"block\">\n" +
                "<div class=\"heading\">������������������ ���������� ���������������� ������</div><p>Lorem ipsum dolor sit amet, consectetuer adipiscing elit, sed diam nonummy nibh euismod tincidunt ut laoreet dolore magna aliquam erat volutpat. Ut wisi enim ad minim veniam, quis nostrud exerci tation ullamcorper suscipit lobortis nisl ut aliquip ex ea commodo consequat. Duis autem vel eum iriure dolor in hendrerit in vulputate velit esse molestie consequat, vel illum dolore eu feugiat nulla facilisis at vero eros et accumsan et iusto odio dignissim qui blandit praesent luptatum zzril delenit augue duis dolore te feugait nulla facilisi.</p>\n" +
                "</div>\n" +
                "<div class=\"block\">\n" +
                "<div class=\"heading\">������������������</div><p>Lorem ipsum dolor sit amet, consectetuer adipiscing elit, sed diam nonummy nibh euismod tincidunt ut laoreet dolore magna aliquam erat volutpat. Ut wisi enim ad minim veniam, quis nostrud exerci tation ullamcorper suscipit lobortis nisl ut aliquip ex ea commodo consequat. Duis autem vel eum iriure dolor in hendrerit in vulputate velit esse molestie consequat, vel illum dolore eu feugiat nulla facilisis at vero eros et accumsan et iusto odio dignissim qui blandit praesent luptatum zzril delenit augue duis dolore te feugait nulla facilisi.<br />Lorem ipsum dolor sit amet, consectetuer adipiscing elit, sed diam nonummy nibh euismod tincidunt ut laoreet dolore magna aliquam erat volutpat. Ut wisi enim ad minim veniam, quis nostrud exerci tation ullamcorper suscipit lobortis nisl ut aliquip ex ea commodo consequat. Duis autem vel eum iriure dolor in hendrerit in vulputate velit esse molestie consequat, vel illum dolore eu feugiat nulla facilisis at vero eros et accumsan et iusto odio dignissim qui blandit praesent luptatum zzril delenit augue duis dolore te feugait nulla facilisi.<br />Lorem ipsum dolor sit amet, consectetuer adipiscing elit, sed diam nonummy nibh euismod tincidunt ut laoreet dolore magna aliquam erat volutpat. Ut wisi enim ad minim veniam, quis nostrud exerci tation ullamcorper suscipit lobortis nisl ut aliquip ex ea commodo consequat. Duis autem vel eum iriure dolor in hendrerit in vulputate velit esse molestie consequat, vel illum dolore eu feugiat nulla facilisis at vero eros et accumsan et iusto odio dignissim qui blandit praesent luptatum zzril delenit augue duis dolore te feugait nulla facilisi.</p>\n" +
                "</div>\n" +
                "<div class=\"block\">\n" +
                "<ul>\n" +
                "<li>������������ ���������� ������ ������������������</li>\n" +
                "<li>�������������� ������������ ���������� ������</li>\n" +
                "<li>�������������� ������������ ���������� ������</li>\n" +
                "<li>�������������� ������������ ���������� ������������</li>\n" +
                "<li>�������������� ������������ ���������� ��������</li>\n" +
                "<li>�������������� ������������ ���������� ����������</li>\n" +
                "</ul>\n" +
                "</div>\n" +
                "<div class=\"block\">\n" +
                "<div class=\"heading\">������������������</div><p>Lorem ipsum dolor sit amet, consectetuer adipiscing elit, sed diam nonummy nibh euismod tincidunt ut laoreet dolore magna aliquam erat volutpat. Ut wisi enim ad minim veniam, quis nostrud exerci tation ullamcorper suscipit lobortis nisl ut aliquip ex ea commodo consequat. Duis autem vel eum iriure dolor in hendrerit in vulputate velit esse molestie consequat, vel illum dolore eu feugiat nulla facilisis at vero eros et accumsan et iusto odio dignissim qui blandit praesent luptatum zzril delenit augue duis dolore te feugait nulla facilisi.</p>\n" +
                "</div>\n" +
                "<div class=\"block\">\n" +
                "<div class=\"heading\">������������������</div><p>Lorem ipsum dolor sit amet, consectetuer adipiscing elit, sed diam nonummy nibh euismod tincidunt ut laoreet dolore magna aliquam erat volutpat. Ut wisi enim ad minim veniam, quis nostrud exerci tation ullamcorper suscipit lobortis nisl ut aliquip ex ea commodo consequat. Duis autem vel eum iriure dolor in hendrerit in vulputate velit esse molestie consequat, vel illum dolore eu feugiat nulla facilisis at vero eros et accumsan et iusto odio dignissim qui blandit praesent luptatum zzril delenit augue duis dolore te feugait nulla facilisi.</p>\n" +
                "</div>\n" +
                "<div class=\"block\">\n" +
                "<div class=\"heading\">������������������</div>\n" +
                "<ul>\n" +
                "<li>�������������� ������������ ���������� ������</li>\n" +
                "<li>�������������� ������������ ���������� ������</li>\n" +
                "<li>�������������� ������������ ���������� ������</li>\n" +
                "<li>�������������� ������������ ���������� ������������</li>\n" +
                "<li>�������������� ������������ ���������� ��������</li>\n" +
                "<li>�������������� ������������ ���������� ����������</li>\n" +
                "</ul>\n" +
                "</div>\n" +
                "<div class=\"block keywords\">\n" +
                "<div class=\"heading\">���������������� ����������</div>\n" +
                "<ul><li><a href=\"#\">���������������� ����������</a>, </li><li><a href=\"#\">���������������� ����������</a>, </li><li><a href=\"#\">���������������� ����������</a>, </li><li><a href=\"#\">���������������� ����������</a>, </li><li><a href=\"#\">���������������� ����������</a>, </li><li><a href=\"#\">���������������� ����������</a>, </li><li><a href=\"#\">���������������� ����������</a>, </li><li><a href=\"#\">���������������� ����������</a></li></ul>\n" +
                "</div>\n" +
                "</div>\n" +
                "</div>\n" +
                " -->";
        
        String expected = "jQuery.noConflict()(function(){ jQuery('#mycarousel').jcarousel({ scroll: 2, animation: 'normal' }); " +
                "jQuery(\"#mycarousel li a\").click(function(){ jQuery(\"#large img\").hide().attr({\"src\": jQuery(this).attr(\"href\"), " +
                "\"title\": jQuery(\"> img\", this).attr(\"title\")}); return false; }); jQuery(\"#large>img\").load(function(){jQuery" +
                "(\"#large>img:hidden\").show(\"fast\")}); jQuery('#popup .close').click(function(){ jQuery(\"#popup\").css('display', 'none'); " +
                "}); }); «Индустрия Чистоты» предлагает своим клиентам следующие виды услуг по профессиональной уборке: " +
                "Ежедневная уборка Генеральная уборка помещений различного назначения Уборка квартир, коттеджей Уборка после пожара " +
                "Послестроительная уборка Химчистка ковров Химчистка мягкой мебели Чистка жалюзи Мытье окон Мытье фасадов " +
                "Глубокая чистка и защита напольных покрытий Чистка саун и бассейнов Ждем Ваших звонков по телефонам:" +
                " 8-901-313-21-32 (круглосуточно), 492-94-18, 492-93-98";       

        String txt = HtmlUtil.html2text(html);
       

        Assert.assertTrue(txt.length() > 0);
        Assert.assertEquals(expected, txt);

        String value = normalizeQueryString(txt);
        value = value.replaceAll("[^а-яА-Яa-zA-Z\\s]", "");
        String[] words = value.split("\\s+");
        ArrayList<String> l = new ArrayList<String>();
        for(String word : words){
            if(word != null && word.length() > 3){
                l.add(word);
            }
        }

        Assert.assertTrue(l.size() > 0);

    }

    private static String normalize(String source) {
        if (source == null) return null;
        return source
                .trim()
                .toLowerCase()
                .replaceAll("ё", "е")
                .replaceAll("[,\":()\\]\\[]", " ")
                .replaceAll("\\s+", " ");
//               .replaceAll(" +", " ");
    }

    private static String cleanAddressLexems(String source) {
        String[] words = sptilOnWords(source);
        StringBuilder out = new StringBuilder(source.length() + words.length);

        for (int i = 0; i < words.length; i++) {
            boolean isLastWord = i == words.length - 1;
            String result = words[i];

            if (result.endsWith(".")) {
                for (Map.Entry<String, String> entry : addressLexems.entrySet()) {
                    String pattern = entry.getKey();
                    if (pattern.equals(result)) {
                        result = entry.getValue();
                        break;
                    }
                }
                out.append(result);
            } else {
                out.append(result);
            }

            if (!isLastWord)
                out.append(" ");
        }
        return out.toString();
    }

    private static Map<String, String> addressLexems;

    static {
        addressLexems = new LinkedHashMap<String, String>();
        addressLexems.put("пер.", "переулок");
        addressLexems.put("ул.", "улица");
        addressLexems.put("ал.", "аллея");
        addressLexems.put("ш.", "шоссе");
        addressLexems.put("м.", "малый");
        addressLexems.put("мал.", "малый");
        addressLexems.put("пл.", "площадь");
        addressLexems.put("туп.", "тупик");
        addressLexems.put("просп.", "проспект");
        addressLexems.put("пр.", "проезд");
        addressLexems.put("поп.", "поперечно");
        addressLexems.put("дер.", "деревня");
        addressLexems.put("бульв.", "бульвар");
        addressLexems.put("бул.", "бульвар");
        addressLexems.put("б.", "большой");
        addressLexems.put("верх.", "верхний");
        addressLexems.put("ниж.", "нижний");
        addressLexems.put("нов.", "новая");
        addressLexems.put("ср.", "средний");
        addressLexems.put("стар.", "старая");
        addressLexems.put("им.", "имени");
        addressLexems.put("наб.", "набережная");
        addressLexems.put("р.", "река");
    }

    /**
     * Split sequence on a words (with terminated punctuation characters)
     * @param input sequernce
     * @return arrat of words
     */
    private static String[] sptilOnWords(CharSequence input) {
        input = SPACES_REMOVER.matcher(input).replaceAll(" ").trim();
        int index = 0;

        ArrayList<String> matchList = new ArrayList<String>();
        Matcher m = WORDS_SPLITTER.matcher(input);

        // Add segments before each match found
        while (m.find()) {
            String match = input.subSequence(index, m.start() + 1).toString().trim();
            matchList.add(match);
            index = m.end();
        }

        // If no match was found, return this
        if (index == 0)
            return new String[]{input.toString()};

        // Add remaining segment
        matchList.add(input.subSequence(index, input.length()).toString());

        // Construct result
        int resultSize = matchList.size();
        while (resultSize > 0 && matchList.get(resultSize - 1).equals(""))
            resultSize--;
        String[] result = new String[resultSize];
        return matchList.subList(0, resultSize).toArray(result);

    }

    private static Pattern WORDS_SPLITTER = Pattern.compile("[\\s,.]+");
    private static Pattern SPACES_REMOVER = Pattern.compile("\\s+");

    private static String normalizeQueryString(String source) {
        String result = normalize(source);
        return cleanAddressLexems(result);
    }

}
