/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.util.mail;

import java.io.IOException;

import javax.mail.MessagingException;

import org.builtu.util.mail.MailUtil;
import org.junit.Assert;
import org.junit.Test;


public class TestMailUtil  {
    
    @Test public void testRandom() throws IOException, MessagingException {
        final String from = "me@somewhere";
        final String to = "someone@anywhere";
        final String subj = "some subject";
        final String body = "body text";
        testSimpleMail(from, to, subj, body);
    }
    
    @Test public void testSimple() throws IOException, MessagingException {
        testSimpleMail("from", "to", "subj", "body");
    }
    
    @Test public void testManyRecepients () throws MessagingException, IOException {
    	final String from = "he@somewhereelse";
    	final String to = "me@somewhere, he@somewhereelse";
    	final String subj = "some subject";
    	final String body = "body text";
    	
        final String text = MailUtil.makeMessage(from, to, subj, body);
        
        Assert.assertTrue(text.indexOf("me@somewhere")>=0);
        Assert.assertTrue(text.indexOf("he@somewhereelse")>=0);
    }
    
    @Test public void testSubjectEncoding() throws IOException, MessagingException {
    	final String from = "he@somewhereelse";
    	final String to = "me@somewhere, he@somewhereelse";
    	final String subj = "Добавить компанию";
    	final String body = "body text";
        
    	final String text = MailUtil.makeMessage(from, to, subj, body);
    	Assert.assertTrue(text.indexOf("Subject: =?UTF-8?")>=0);
    	
    }
    

    /*@Test public void testRussian() throws IOException, MessagingException {
        testSimpleMail("для", "от", "сабж", "тело");
    }*/
    
    private void testSimpleMail(
            final String from, 
            final String to,
            final String subj,
            final String body
    ) throws IOException, MessagingException {
        final String text = MailUtil.makeMessage(from, to, subj, body);
        Assert.assertTrue(text.indexOf(from)>=0);
        Assert.assertTrue(text.indexOf(to)>=0);
        Assert.assertTrue(text.indexOf(subj)>=0);
        Assert.assertTrue(text.indexOf(body)>=0);
    }
}
