package org.builtu.webutil.ordreq;

import java.lang.Thread.State;

import junit.framework.Assert;

import org.junit.Before;
import org.junit.Test;

public class TestOrdreqLock {

	private OrdreqLock lock;
	
	@Before public void setUp()  {
		lock = new OrdreqLock();
	}
	
	@Test public void testSimple() throws InterruptedException {
		lock.lockInterruptibly("1");
		lock.unlock("1");
	}
	
	@Test public void testReentrant() throws InterruptedException {
		lock.lockInterruptibly("1");
		lock.lockInterruptibly("1");
		lock.unlock("1");
		lock.unlock("1");
	}
	
	@Test public void testReentrantThread() throws InterruptedException {
		lock.lockInterruptibly("1");
		lock.lockInterruptibly("2");
		lock.unlock("2");
		lock.unlock("1");
	}
	
	@Test public void testBlockInterruptiblyThread() throws InterruptedException {
		lock.lockInterruptibly("1");
		final Thread t = new Thread(new Runnable() {
			@Override public void run() {
				try {
					lock.lockInterruptibly("1");
				} catch (InterruptedException e) {
					throw new RuntimeException(e);
				}
				lock.unlock("1");
			}});
		t.start();
		Thread.sleep(10);
		Assert.assertEquals(State.WAITING, t.getState());
		lock.unlock("1");
		t.join();
	}

	@Test public void testDontBlockInterruptiblyOtherId() throws InterruptedException {
		lock.lockInterruptibly("1");
		final Thread t = new Thread(new Runnable() {
			@Override public void run() {
				try {
					lock.lockInterruptibly("2");
				} catch (InterruptedException e) {
					throw new RuntimeException(e);
				}
				lock.unlock("2");
			}});
		t.start();
		t.join();
	}
	
}
