package org.builtu;

import java.util.Date;

import junit.framework.Assert;

import org.junit.Before;
import org.junit.Test;

public class TestAggregator {
	
	protected static class SimpleListener implements StatListener {
		public Date from, to;
		int lastCount, maxCount, called;
		
		@Override public void listen(Date from, Date to, int liveSessionsCount) {
			if (this.from == null) this.from = from;
			this.to = to;
			
			this.lastCount = liveSessionsCount;
			if (this.maxCount < liveSessionsCount) this.maxCount = liveSessionsCount;
			
			this.called ++;
		}
	}
	
	protected Aggregator aggregator;
	protected SimpleListener listener;
	
	@Before public void createAggregator() {
		aggregator = new Aggregator();
		aggregator.setInterval(1000000); // prevents aggregator from calling listeners
		listener = new SimpleListener();
		aggregator.listeners.add(listener);
	}
	
	@Test public void testOneRecord() {
		aggregator.setInterval(0);
		aggregator.keepAlive("session");
		aggregator.keepAlive("session");
		aggregator.commit();
		Assert.assertEquals(1, listener.lastCount);
		Assert.assertTrue("from="+listener.from+", to="+listener.to, listener.from.compareTo(listener.to)<=0);
	}
	
	@Test public void testDelayedNotification () {
		aggregator.keepAlive("session");
		Assert.assertEquals(0, listener.lastCount);
		aggregator.setInterval(0);
		aggregator.keepAlive("session");
		Assert.assertEquals(1, listener.lastCount);
	}
	
	@Test public void testTwoSessions() {
		aggregator.keepAlive("session1");
		aggregator.setInterval(0);
		aggregator.keepAlive("session2");
		Assert.assertEquals(2, listener.lastCount);
	}
	
	@Test public void testExpireSession() throws InterruptedException {
		aggregator.setSessionTimeout(1);
		aggregator.keepAlive("expired_session");
		Thread.sleep(2);
		aggregator.setInterval(0);
		aggregator.keepAlive("session2");
		Assert.assertEquals(1, listener.lastCount);
	}
}
